import {
  select,
  selectAll,
  scaleOrdinal
} from 'd3';

const svg = select('svg');
const width = +svg.attr('width'); 
const height = +svg.attr('height');
const centre_x = width / 2;
const centre_y = height / 2;
const colorScale = ["red","green","blue", "gold", "orange", "none"];

svg.attr("preserveAspectRatio", "xMidYMid meet")
   .attr("viewBox", [0, 0, width * 1.2, height * 1.2]);

const defs = svg.append("defs");
		defs.selectAll("marker")
			.data(["node"])
			.enter()
			.append("marker")
				.attr("id", function (d) { return d; })
				.attr("viewBox", "0 -2.5 5 5")
				.attr("refX", 5)
				.attr("refY", 0)
				.attr("markerWidth", 5)
				.attr("markerHeight", 5)
				.attr("orient", "auto")
			.append("path")
				.attr("d", "M0,-2.5L5,0L0,2.5")
				.attr("fill", "#999");

d3.json('data.json')
  .then(data => {
  
  const links = data.links.map(d => Object.create(d));
  links.forEach((d, i) => {
//    console.log(data.links[i]);
    d.srcType = data.links[i].srcType;
    d.tgtType = data.links[i].tgtType;
    d.relationship = data.links[i].relationship;
    d.value = data.links[i].value;
  });
  const nodes = data.nodes.map(d => Object.create(d));
  nodes.forEach((d, i) => {
  	d.group = data.nodes[i].group;
  });
  
  const simulation = d3.forceSimulation(nodes)
      .force("link", d3.forceLink(links)
             .id(d => d.id)
             .distance(90))
      .force("charge", d3.forceManyBody()
             .strength(-350)
             .distanceMax(150))
  		.force("collision", d3.forceCollide().radius(25))
      .force("center", d3.forceCenter(centre_x, centre_y))
  ;
  
  const drag = simulation => {
    function dragstarted(event) {
      if (!event.active) simulation.alphaTarget(0.3).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
    }
    function dragged(event) {
      event.subject.fx = event.x;
      event.subject.fy = event.y;
    }
    function dragended(event) {
      if (!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
   //   event.subject.fixed = true;
      console.log(event);
    }
    return d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended);
  }

  const link = svg.append("g")
  .selectAll("line")
    .data(links)
  	.join("line")
  		.attr("marker-end", "url(#node)");
  
  link.append("text")
  			.attr("dy", "-0.3em")
  		.append("textpath")
  			.text(d => d.relationship
              ? d.relationship.length > 4
              	? d.relationship.slice(0, 4) + "..."
              	: d.relationship
              : null)
  			.attr("x", d => d.source.x)
  			.attr("y", d => d.source.y);
  
  const node = svg.append("g")
    .attr("class", "node")
  .selectAll("circle")
    .data(nodes)
  	.join("circle")
  		.attr("r", 20)
  		.attr("fill", d => colorScale[d.group])
  		.call(drag(simulation));
  
  const label = node.append("text")
  			.text(d => d.id)
  			.attr("x", 12)
  			.attr("y", -12)
  			.attr("dy", "-0.5em");
  
 	simulation.on("tick", () => {
    link
 			.attr("x1", d => d.source.x)
 			.attr("y1", d => d.source.y)
 			.attr("x2", d => d.target.x)
 			.attr("y2", d => d.target.y);
    node
 			.attr("cx", d => d.x)
 			.attr("cy", d => d.y);
  });
  
});
